/*
 * This file is part of the MDR library.
 * Copyright (C) 2013-2023 Mateusz Viste
 */

#include <i86.h>

#include <mdr\cout.h>


void mdr_coutraw_crlf(void) {
  _asm {
    /* output a CR/LF terminator */
    mov ah, 2
    mov dl, 13
    int 0x21
    mov dl, 10
    int 0x21
  }
}


void mdr_coutraw_str(const char *s) {
  unsigned short s_seg = FP_SEG(s);
  unsigned short s_off = FP_OFF(s);

  _asm {
    push ds
    push es

    /* set DS and ES to source string */
    mov ax, s_seg
    mov ds, ax
    mov es, ax

    /* compute strlen of s and put it in cx */
    mov di, s_off
    xor al, al /* look for 0 char */
    xor cx, cx
    not cx    /* load cx with 0xffff (maximum loop iterations) */
    cld       /* clear direction flag so repnz is guaranteed to move forward */
    repne scasb
    not cx
    dec cx    /* cx contains strlen(s) now */

    /* ah=0x40 -- "write to file or device" */
    mov ah, 0x40
    mov dx, s_off    /* source string */
    xor bx, bx       /* file handle 0 is stdout */
    /* mov cx, slen */ /* bytes count */
    int 0x21

    pop es
    pop ds
  }
}


void mdr_coutraw_puts(const char *s) {
  mdr_coutraw_str(s);
  mdr_coutraw_crlf();
}
