/*
 * EXEPATH - returns the directory from which the program was loaded.
 *
 * This file is part of the Mateusz' DOS Routines (MDR): http://mdr.osdn.io
 * Published under the terms of the MIT License, as stated below.
 *
 * Copyright (C) 2014-2023 Mateusz Viste
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <mdr\dos.h>

/* fetches directory where the program was loaded from and return its length.
 * path string is never longer than 128 (incl. the null terminator) and it is
 * always terminated with a backslash separator, unless it is an empty string */
unsigned char mdr_dos_exepath(char *path) {
  const char far *env;
  unsigned int i;
  int lastsep;

  /* get the full path and filename of running program */
  env = mdr_dos_selfexe();
  if (env == NULL) {
    path[0] = 0;
    return(0);
  }

  /* copy the EXEPATH to result and truncate after last '\' */
  lastsep = -1;
  for (i = 0;; i++) {
    path[i] = *env;
    if (path[i] == '\\') lastsep = i;
    if (path[i] == 0) break; /* end of string */
    if (i >= 126) {
      lastsep = -1;
      break;     /* this DOS string should never go beyond 127 chars! */
    }
    env++;
  }
  lastsep++;
  path[lastsep] = 0;
  return(lastsep);
}
