/*
 * This file is part of the Mateusz' DOS Routines (MDR): http://mdr.osdn.io
 * Published under the terms of the MIT License, as stated below.
 *
 * Copyright (C) 2014-2023 Mateusz Viste
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <i86.h> /* MK_FP() */

#include <mdr\dos.h>

/* looks for varname in the DOS environment and fills result with its value if
 * found. returns NULL if not found or if value is too big to fit in result
 * (reslimit exceeded). returns result on success.
 * NOTE: this function performs case-sensitive matches */
char *mdr_dos_getenv(char *result, const char *varname, unsigned short reslimit) {
  unsigned short i;
  char far *envptr = mdr_dos_env();

  while (*envptr != 0) {
    for (i = 0;; i++) {
      if ((varname[i] == 0) && (envptr[i] == '=')) {
        /* MATCH FOUND: copy the value to result, up to reslimit bytes */
        envptr += i + 1;
        i = 0;
        for (;;) {
          if (reslimit == i) return(NULL); /* not enough space in result */
          result[i] = envptr[i];
          if (result[i] == 0) {
            return(result);
          }
          i++;
        }
      }
      if ((varname[i] != envptr[i]) || (varname[i] == 0)) break;
    }
    /* fast-forward to next variable */
    envptr += i;
    while (*envptr != 0) envptr++;
    envptr++;
  }
  return(NULL);
}
