/*
 * Keyboard routines
 *
 * This file is part of the Mateusz' DOS Routines (MDR): http://mdr.osdn.io
 * Published under the terms of the MIT License, as stated below.
 *
 * Copyright (C) 2014-2024 Mateusz Viste
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <mdr\dos.h>  /* include self for control */

/* waits for a keypress and return it. Extended keys are returned ORed with
 * 0x100 (example: PGUP is 0x149). This call is immune to CTRL+C. */
static int internal_getkey2(void);

#pragma aux internal_getkey2 = \
"xor bl, bl" \
"mov ah, 0x07" /* AH=07h is DOS 1+ "DIRECT CHARACTER INPUT W/O ECHO" */ \
"int 0x21" \
"test al, al" \
"jnz done" \
"inc bl" \
"int 0x21" \
"done:" \
"mov ah, bl" \
modify [bl] \
value [ax];

int mdr_dos_getkey2(void) {
  return(internal_getkey2());
}
