/*
 * TRUENAME
 *
 * This file is part of the Mateusz' DOS Routines (MDR): http://mdr.osdn.io
 * Published under the terms of the MIT License, as stated below.
 *
 * Copyright (C) 2014-2023 Mateusz Viste
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <i86.h> /* FP_SEG(), FP_OFF() */

#include <mdr/dos.h> /* include self */


void mdr_dos_truename(char *result, const char *name) {
  unsigned short result_off = FP_OFF(result);
  unsigned short result_seg = FP_SEG(result);
  unsigned short name_off = FP_OFF(name);
  unsigned short name_seg = FP_SEG(name);

  _asm {
    push es
    push ds

    /* es:di = result buffer */
    mov es, result_seg
    mov di, result_off
    /* ds:si = ASCIZ filename or path */
    mov si, name_off
    mov ax, name_seg
    mov ds, ax

    cld /* make sure that lodsb/stosb move forward */

    /* copy filename as-is in case function 21h,ah=60h is not available */
    NEXTBYTE:
    /* load DS:SI to AL and SI++ */
    lodsb
    /* write AL to ES:DI and DI++ */
    stosb
    /* are we done? (look for the C-style nul terminator) */
    test al, al
    jnz NEXTBYTE

    /* reset SI and DI and call the truename function */
    mov di, result_off
    mov si, name_off

    mov ah, 0x60  /* DOS 3+ - canonicalize filename or path */
    int 0x21

    pop ds
    pop es
  }
}
