/*
 * Mouse routines
 *
 * This file is part of the Mateusz' DOS Routines (MDR): http://mdr.osdn.io
 * Published under the terms of the MIT License, as stated below.
 *
 * Copyright (C) 2014-2023 Mateusz Viste
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <i86.h>
#include <mdr/mouse.h>  /* include self for control */

/* set graphic pointer shape. icon is 64-bytes long, two sets of 32. each set
 * of 32 bytes is organized as a 16x16 bitmap, ie. 16 rows of 16-bit values.
 * a) the first set of 16 shorts defines the background mask - that is, the
 * background will show through wherever there is a 1-bit in that data.
 * b) the second set defines the "XOR mask" - that is, the pixels matching the
 * 1-bit in this data set are toggled.
 * hotspotx and hotspoty define respectively the horizontal and vertical hot
 * spot of the pointer (default being [0,0], that is the top left corner). */
void mdr_mouse_setcursor(const unsigned short *icon, unsigned char hotspotx, unsigned char hotspoty) {
  unsigned short icon_seg = FP_SEG(icon);
  unsigned short icon_off = FP_OFF(icon);
  _asm {
    push es
    mov ax, 0x09
    xor bx, bx
    xor cx, cx
    mov bl, hotspotx
    mov cl, hotspoty
    mov dx, icon_off
    mov es, icon_seg;
    int 0x33
    pop es
  }
}
