/*
 * OPL2 timbre-building function
 *
 * This file is part of the Mateusz' DOS Routines (MDR): http://mdr.osdn.io
 * Published under the terms of the MIT License, as stated below.
 *
 * Copyright (C) 2015-2023 Mateusz Viste
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <mdr/opl.h>

/* generate a timbre structure based on a timbre template. this is a
 * convenience function meant to provide a human-compatible (more readable)
 * way of generating a timbre struct. */
int mdr_opl_timbre_gen(struct mdr_opl_timbre *timbre, const struct mdr_opl_timbretemplate *tpl) {
  /* wave selector */
  timbre->car_ws = tpl->carrier.ws;
  timbre->mod_ws = tpl->modultr.ws;

  /* sustain / release */
  timbre->car_sr = (tpl->carrier.sustain << 4) | tpl->carrier.release;
  timbre->mod_sr = (tpl->modultr.sustain << 4) | tpl->modultr.release;

  /* attack / decay */
  timbre->car_ad = (tpl->carrier.attack << 4) | tpl->carrier.decay;
  timbre->mod_ad = (tpl->modultr.attack << 4) | tpl->modultr.decay;

  /* Tremolo / Vibrato / Sustain / KSR / Frequency Multiplication Factor */
  timbre->car_20 = (tpl->carrier.tremolo << 7) | (tpl->carrier.vibrato << 6) | (tpl->carrier.sustain << 5) | (tpl->carrier.ksr << 4) | (tpl->carrier.mult);
  timbre->mod_20 = (tpl->modultr.tremolo << 7) | (tpl->modultr.vibrato << 6) | (tpl->modultr.sustain << 5) | (tpl->modultr.ksr << 4) | (tpl->modultr.mult);

  /* Key Scale Level / Output Level */
  timbre->car_40 = (tpl->carrier.ksl << 6) | tpl->carrier.outlev;
  timbre->mod_40 = (tpl->modultr.ksl << 6) | tpl->modultr.outlev;

  /* Feedback / Connection */
  timbre->feedconn = (tpl->feedback << 1) | tpl->conn;

  return(0);
}
