/*
 * PCX-loading routines
 *
 * This file is part of the Mateusz' DOS Routines (MDR): http://mdr.osdn.io
 * Published under the terms of the MIT License, as stated below.
 *
 * Copyright (C) 2022-2023 Mateusz Viste
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <mdr/pcx.h>

/* load an entire PCX file into a pixel buffer. the PCX data must have been
 * previously analyzed by pcx_anal() and the fd file pointer must not have been
 * modified since then. the destination buffer must be large enough to hold all
 * pixels, ie. (h->max_x + 1) * (h->max_y + 1) for 8 bpp.
 * returns 0 on success, non-zero otherwise. */
int mdr_pcx_load(void *pixbuf, const struct pcx_hdr *h, FILE *fd) {
  unsigned short row;
  int r;

  for (row = 0; row <= h->max_y; row++) {
    r = mdr_pcx_loadrow(pixbuf, h, fd);
    pixbuf = (char *)pixbuf + h->bytes_per_scanline;
    if (r != 0) return(r);
  }

  return(0);
}
