/*
 * PCX-loading routines
 *
 * This file is part of the Mateusz' DOS Routines (MDR): http://mdr.osdn.io
 * Published under the terms of the MIT License, as stated below.
 *
 * Copyright (C) 2022-2023 Mateusz Viste
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <mdr/pcx.h>

/* convert img to 8bpp if needed (ie unpack 1, 2 or 4bpp into 8bpp) */
int mdr_pcx_to8bpp(void *img, const struct pcx_hdr *h, unsigned char rowflag) {
  unsigned char *imgptr;
  unsigned char *pix;
  unsigned short imglen;

  /* validate parameters */
  if ((img == NULL) || (h == NULL)) return(-1);

  switch (h->bpp) {
    case 8:
      return(0);
    case 4:
    case 2:
    case 1:
      break;
    default:
      return(-1);
  }

  imglen = h->max_x + 1;
  if (rowflag == 0) imglen *= h->max_y + 1;
  pix = (char *)img + imglen - 1; /* last pixel of target (8 bpp) image */
  imglen /= (8 / h->bpp);

  /* start at last pixel so I can convert all in place */
  imgptr = (char *)img + imglen - 1;

  switch (h->bpp) {
    case 4: /* 4 to 8 bpp */
      for (;;) {
        *pix = *imgptr & 0x0f;
        pix--;
        *pix = (*imgptr >> 4) & 0x0f;
        pix--;
        if (imgptr == img) break;
        imgptr--;
      }
      break;

    case 2: /* 2 to 8 bpp */
      for (;;) {
        *pix = *imgptr & 3;
        pix--;
        *pix = (*imgptr >> 2) & 3;
        pix--;
        *pix = (*imgptr >> 4) & 3;
        pix--;
        *pix = (*imgptr >> 6) & 3;
        pix--;
        if (imgptr == img) break;
        imgptr--;
      }
      break;

    case 1: /* 1 to 8 bpp */
      for (;;) {
        *pix = *imgptr & 1;
        pix--;
        *pix = (*imgptr >> 1) & 1;
        pix--;
        *pix = (*imgptr >> 2) & 1;
        pix--;
        *pix = (*imgptr >> 3) & 1;
        pix--;
        *pix = (*imgptr >> 4) & 1;
        pix--;
        *pix = (*imgptr >> 5) & 1;
        pix--;
        *pix = (*imgptr >> 6) & 1;
        pix--;
        *pix = (*imgptr >> 7) & 1;
        pix--;
        if (imgptr == img) break;
        imgptr--;
      }
      break;
  }

  return(0);
}
