/*
 * test application for trigint
 * modern platforms only, as it requires SDL2
 */

#include <math.h>
#include <stdio.h>
#include <SDL2/SDL.h>        /* SDL */

#include "trigint.h"


int calcerr(float trueval, short intval) {
  return(fabs(fabs(trueval * 1000) - fabs(intval)));
}


int main(void) {
  int x, y;
  float s, c, rad;
  int sinerr, coserr;
  short icos, isin;
  char coserrstr[16], sinerrstr[16];

  SDL_Renderer *renderer;
  SDL_Window *window;

  /* init the SDL library */
  SDL_Init(SDL_INIT_VIDEO);
  SDL_CreateWindowAndRenderer(640, 480, 0, &window, &renderer);
  SDL_RenderClear(renderer);

  for (x = 0; x < 640; x++) {
    rad = x * 3.14159265 / 180;

    s = sin(rad);
    c = cos(rad);

    isin = trigint_sin(rad * 1000);
    icos = trigint_cos(rad * 1000);

    sinerr = calcerr(s, isin);
    coserr = calcerr(c, icos);
    sinerrstr[0] = 0;
    coserrstr[0] = 0;
    if (sinerr > 2) sprintf(sinerrstr, " [ERR=%d]", sinerr);
    if (coserr > 2) sprintf(coserrstr, " [ERR=%d]", coserr);

    printf("sin %d deg (%f rad) = %0.3f (intsin=%d)%s\n", x, rad, s, isin, sinerrstr);

    SDL_SetRenderDrawColor(renderer, 0, 255, 0, 255);
    SDL_RenderDrawPoint(renderer, x, 120 + s * 100);
    SDL_SetRenderDrawColor(renderer, 255, 0, 0, 255);
    SDL_RenderDrawPoint(renderer, x, 125 + isin / 10);

    printf("cos %d deg (%f rad) = %0.3f (intcos=%d)%s\n", x, rad, c, icos, coserrstr);
    SDL_SetRenderDrawColor(renderer, 0, 255, 0, 255);
    SDL_RenderDrawPoint(renderer, x, 350 + c * 100);
    SDL_SetRenderDrawColor(renderer, 255, 0, 0, 255);
    SDL_RenderDrawPoint(renderer, x, 355 + icos / 10);
  }

  for (rad = -3.14159; rad < 3.14159; rad += 0.01) {
    x = cos(rad) * 236;
    y = sin(rad) * 236;
    SDL_SetRenderDrawColor(renderer, 0, 255, 0, 255);
    SDL_RenderDrawPoint(renderer, 320 + x, 240 + y);

    x = trigint_cos(rad * 1000) * 10 / 43;
    y = trigint_sin(rad * 1000) * 10 / 43;
    SDL_SetRenderDrawColor(renderer, 255, 0, 0, 255);
    SDL_RenderDrawPoint(renderer, 320 + x, 240 + y);

  }

  for (;;) {
    SDL_Event event;
    SDL_RenderPresent(renderer);
    if (SDL_WaitEvent(&event)) {
      if (event.type == SDL_QUIT) break;
    }
   }

  /* clean up SDL */
  SDL_DestroyRenderer(renderer);
  SDL_DestroyWindow(window);
  SDL_Quit();

  return(0);
}
