/*
 * video library - provides a few functions for mode 4 and 13h programming.
 *
 * This file is part of the Mateusz' DOS Routines (MDR): http://mdr.osdn.io
 * Published under the terms of the MIT License, as stated below.
 *
 * Copyright (C) 2014-2022 Mateusz Viste
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <string.h>    /* memcpy() */

#include <mdr\video.h> /* include self for control */


/* update the screen (if double-buffered) */
void video_flip(struct video_handler *handler) {
  unsigned char far *vram;
  /* copy screen buffer to vram only if we are using double buffering */
  if ((handler->flags & VIDEO_DBUF) == 0) return;
  if (handler->mode == 0x13) {
    vram = (unsigned char far*)0xA0000000l;
    _fmemcpy(vram, handler->dbuf, 64000u);
  } else if (handler->mode == 0x04) {
    vram = (unsigned char far*)0xB8000000l;
    _fmemcpy(vram, handler->dbuf, 8000);
    _fmemcpy(vram + 8192, handler->dbuf + 8192, 8000);
  }
}
