/*
 * video library - provides a few functions for mode 4 and 13h programming.
 *
 * This file is part of the Mateusz' DOS Routines (MDR): http://mdr.osdn.io
 * Published under the terms of the MIT License, as stated below.
 *
 * Copyright (C) 2014-2022 Mateusz Viste
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <mdr\video.h> /* include self for control */

/* renders a sprite of width and height dimensions onscreen, starting at specified x/y location
   coloffset is an offset to add to color indexes, while transp is the index of the transparent color (set to -1 if none) */
void video_putsprite(struct video_handler *handler, unsigned char *sprite, int x, int y, int width, int height, int coloffset, int transp, int maxcol) {
  int pixelx, pixely, pixelval;
  if ((maxcol > 255) || (maxcol < 0)) maxcol = 255;
  for (pixely = 0; pixely < height; pixely++) {
    for (pixelx = 0; pixelx < width; pixelx++) {
      pixelval = sprite[pixely * width + pixelx];
      if (handler->mode == 0x04) if (pixelval < 7) pixelval = 0; else pixelval = transp; /* nasty hack for antialiased text */
      if (pixelval == transp) continue;
      if (pixelval > maxcol) pixelval = maxcol;
      video_rputpixel(handler, x + pixelx, y + pixely, pixelval + coloffset, 1);
    }
  }
}
