/*
 * video library - provides a few functions for mode 4 and 13h programming.
 *
 * This file is part of the Mateusz' DOS Routines (MDR): http://mdr.osdn.io
 * Published under the terms of the MIT License, as stated below.
 *
 * Copyright (C) 2014-2022 Mateusz Viste
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <stdio.h>     /* FILE */
#include <stdlib.h>    /* malloc() */

#include <mdr\video.h> /* include self for control */


/* uncompress a RLE stream from a file into a buffer */
static void video_rlefile2buff(char *file, unsigned char far *buff, long foffset, long explen) {
  unsigned char rlebitlen, bytebuff, rlebitmask;
  unsigned int uncompressedbytes = 0;
  int rle;
  FILE *fd;
  fd = fopen(file, "rb");
  if (fd == NULL) return;
  if (foffset > 0) fseek(fd, foffset, SEEK_SET);
  /* read and validate the RLEBITLEN parameter */
  fread(&rlebitlen, 1, 1, fd);
  if ((rlebitlen > 7) || (rlebitlen < 1)) {
    fclose(fd);
    return;
  }
  rlebitmask = 255;
  rlebitmask <<= rlebitlen;
  rlebitmask &= 0xff;
  /* decompress the stream */
  for (;;) {
    fread(&bytebuff, 1, 1, fd);
    if ((bytebuff & rlebitmask) != rlebitmask) { /* if it's a raw byte, save it once */
        rle = 1;
      } else { /* if it's a RLE marker, read the next byte */
        rle = (bytebuff ^ rlebitmask) + 1; /* reset RLE marker bits and increment */
        fread(&bytebuff, 1, 1, fd);
    }
    for (; rle > 0; rle--) {
      buff[uncompressedbytes++] = bytebuff;
      if (uncompressedbytes >= explen) break;
    }
    if (uncompressedbytes >= explen) break;
  }
  fclose(fd);
}


/* same as video_putsprite(), but reads the sprite from a file */
void video_putspritefromfile(struct video_handler *handler, char *file, long foffset, int x, int y, int width, int height, int coloffset, int transp, int maxcol) {
  unsigned char *buff;
  buff = malloc(width * height);
  if (buff == NULL) return;
  video_rlefile2buff(file, buff, foffset, width * height);
  video_putsprite(handler, buff, x, y, width, height, coloffset, transp, maxcol);
  free(buff);
}


/* reads a screen dump from file and puts it to the screen buffer */
void video_file2screen(struct video_handler *handler, char *file) {
  video_rlefile2buff(file, handler->dbuf, 0, 64000u);
}
