#include <stdio.h>

#include <mdr\vid12.h>


/* get number of ticks from the BIOS 18.2 Hz timer */
static unsigned long getticks(void) {
  unsigned short ticksh = 0, ticksl = 0;
  unsigned long res;
  _asm {
    push cx
    push dx

    xor ah, ah
    int 0x1A /* returns number of clock ticks since midnight in CX:DX */
    mov ticksh, cx
    mov ticksl, dx

    pop dx
    pop cx
  }
  res = ticksh;
  res <<= 16;
  res |= ticksl;
  return(res);
}


static void test_putpixel(void) {
  #define BLOCKSZ 20
  unsigned short i, x, y;

  for (i = 0; i < 16; i++) {
    unsigned short st = i * BLOCKSZ;
    unsigned short en = st + BLOCKSZ;
    for (y = st; y < en; y++) {
      for (x = st; x < en; x++) {
        vid12_putpixel(x, y, i);
      }
    }
  }
}


static void test_putscanline(void) {
  unsigned short x, y;
  unsigned char scanlinebuf[640];
  /* second test - write scanlines */
  for (y = 0; y < 480; y++) {
    for (x = 0; x < y; x++) {
      scanlinebuf[x] = 0;
    }
    for (; x < 640; x++) {
      scanlinebuf[x] = (x-y) / 40;
    }
    vid12_putscanline(y, scanlinebuf);
  }
}


static void test_rect(void) {
  unsigned short x, y, i;
  unsigned short xstart = 60;
  unsigned short ystart = 240;
  unsigned short l = 80;

  for (i = 1; i < 16; i++) {
    for (x = xstart; x < xstart + l; x++) {
      vid12_putpixel(x, ystart, i);
      vid12_putpixel(x, ystart + l, i);
    }
    for (y = ystart; y <= ystart + l; y++) {
      vid12_putpixel(xstart, y, i);
      vid12_putpixel(xstart + l, y, i);
    }

    xstart += 2;
    ystart += 2;
    l -= 4;
  }
}


static void test_pattern(void) {
  unsigned short x, y;
  for (y = 340; y < 420; y += 2) {
    for (x = 60; x < 140; x += 2) {
      vid12_putpixel(x, y, 15);
    }
  }
}


static void test_scanlinecpy(void) {
  unsigned short i;
  vid12_vramcpy_prep();
  for (i = 285; i < 480; i++) vid12_vramcpy_scanline(i, 284);
  vid12_vramcpy_done();
}


static void test_clrline(void) {
  unsigned short i;
  for (i = 0; i < 480; i++) vid12_clrline(i, i & 15);
}


static char getkey(void) {
  unsigned char k = 0;
  _asm {
    mov ah, 0x08
    int 0x21
    mov k, al
  }
  return(k);
}


int main(void) {
  unsigned long ticks_start;
  unsigned long ticks_cls;
  unsigned long ticks_scanline;
  unsigned long ticks_putpixel;
  unsigned long ticks_scanlinecpy;
  unsigned long ticks_clrline;

  vid12_init();

  vid12_setpalette(0, 0, 0, 0);
  vid12_setpalette(1, 63, 0, 0);
  vid12_setpalette(2, 0, 63, 0);
  vid12_setpalette(3, 0, 0, 63);
  vid12_setpalette(4, 63, 63, 63);

  ticks_start = getticks();
  vid12_cls(0);
  ticks_cls = getticks() - ticks_start;

  ticks_start = getticks();
  test_putscanline();
  ticks_scanline = getticks() - ticks_start;

  ticks_start = getticks();
  test_scanlinecpy();
  ticks_scanlinecpy = getticks() - ticks_start;

  ticks_start = getticks();
  test_putpixel();
  test_rect();
  test_pattern();
  ticks_putpixel = getticks() - ticks_start;

  getkey();

  ticks_start = getticks();
  test_clrline();
  ticks_clrline = getticks() - ticks_start;

  getkey();
  vid12_close();

  printf("cls      = %lu ticks\r\n", ticks_cls);
  printf("scanline = %lu ticks\r\n", ticks_scanline);
  printf("putpixel = %lu ticks\r\n", ticks_putpixel);
  printf("linecpy  = %lu ticks\r\n", ticks_scanlinecpy);
  printf("clrline  = %lu ticks\r\n", ticks_clrline);

  getkey();

  return(0);
}
