/*
 * a few functions for mode 12h programming (640x480 4bpp)
 *
 * This file is part of the Mateusz' DOS Routines (MDR): http://mdr.osdn.io
 * Published under the terms of the MIT License, as stated below.
 *
 * Copyright (C) 2022 Mateusz Viste
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <mdr\vid12.h>

#define VGA ((unsigned char far *)(0xA0000000l))

void vid12_vramcpy_prep(void) {
  /* this relies on the 'VGA latches trick', where reading any byte from VRAM
   * actually reads bytes from all planes and loads the VGA latches with the
   * read values (while the value returned to CPU is only the selected one).
   * then, writing a byte actually writes the content of all latches. */
  _asm {
    /* set write mask to "all" */
    mov al, 2
    mov ah, 15  /* ah is the plane mask (15 = planes 0,1,2,3 enabled) */
    mov dx, 0x3c4
    out dx, ax

    /* set AND operation with latch */
    mov dx, 0x3ce
    mov al, 3 /* index */
    mov ah, 8 /* CPU data is ANDed with latched data */
    out dx, ax
  }
}


void vid12_vramcpy_scanline(unsigned short dst, unsigned short src) {
  unsigned short i;
  /* */
  src *= 80;
  dst *= 80;
  for (i = 0; i < 80; i++) {
    unsigned char b = VGA[src + i]; /* initiate a read pulse to load the latches */
    VGA[dst + i] = b | 0xff; /* 0xff would be enough but then compiler might optimize out the read above */
  }
}


void vid12_vramcpy_done(void) {
  _asm {
    push dx
    mov dx, 0x3ce
    mov al, 3 /* index */
    mov ah, 0 /* CPU data is to be written as-is (latches ignored) */
    out dx, ax
    pop dx
  }
}
